@@ -6,11 +6,12 @@ import subprocess
 import sys
 import traceback
 from types import FrameType
+from typing import Optional
 
 from zulip import RandomExponentialBackoff
 
 
-def die(signal: int, frame: FrameType) -> None:
+def die(signal: int, frame: Optional[FrameType]) -> None:
     """We actually want to exit, so run os._exit (so as not to be caught and restarted)"""
     os._exit(1)
 
